#include "config.h"
#include <sys/types.h>
#include <stdio.h>
#include "ldct-ht.h"

extern u_char MSB[];

/* 
 * Format of ldcttab:
 * 31   29                  19                  9                 0
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |   |   |O| N3| ref3|rl3|   |O| N2| ref2|rl2|   |O| N1| ref1|rl1|
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 * rlX = delta of run length
 * refX = refinement bits (if any)
 * NX = Number of refinement bits
 * O = output the code
 */

u_int
layer_entry(int level, int len, int c)
{
	int m = MSB[c];
	int rlen = 0;
	int outcode = 0;
	/* Run lengths */
	if (c != 0) {
		if (m < level)
			rlen = len;
		else if (m < level + len) {
			/* Bits following MSB in layer */
			int post = m - level;	
			
			rlen = len - post - 1;
			outcode = 1;
		}
	} else
		rlen = len;

	/* Refinement */
	int ref = 0;
	int nref = 0;
	if (c != 0) {
		if (m > level) {
			if (m < level + len) {
				int post = m - level;
				ref = (c >> level) & ((1 << post) - 1);
				nref = post;
			} else {
				ref = (c >> level) & ((1 << len) - 1);
				nref = len;
			}
		}
	}
	
	return ((outcode << 7) | (nref << 5) | (ref << 2) | rlen);
}

int main()
{
	printf("#include <sys/types.h>\n");
        printf("#include \"config.h\"\n\n");
	printf("const u_int ldcttab[] = {\n");
	int c;
	for (c = 0; c < 512; c += 4) {
		u_int o = 0;

		o |= layer_entry(LDCT_L0, LDCT_L0_LEN, c);
		o |= layer_entry(LDCT_L1, LDCT_L1_LEN, c) << 10;
		o |= layer_entry(LDCT_L2, LDCT_L2_LEN, c) << 20;
		printf("0x%x,\n", o);
	}
	printf("};\n");
        return 0;
}

			
			
		
		

